@@ -4,10 +4,11 @@ Run a development server for the Monopyly app.
 import os
 from flask import Flask
 
-import monopyly.db as db
+from monopyly.db import db
 from monopyly.core import core
 from monopyly.auth import auth
 from monopyly.credit import credit
+from monopyly.banking import banking
 
 
 def create_app(test_config=None):
@@ -40,6 +41,9 @@ def create_app(test_config=None):
     # Register the authentication blueprint
     app.register_blueprint(auth)
 
+    # Register the banking financials blueprint
+    app.register_blueprint(banking)
+
     # Register the credit card financials blueprint
     app.register_blueprint(credit)
 
